//
//  ViewController.swift
//  MVisaSDKPrototypeSwift
//
//

import UIKit
import MVisaSDK

class ViewController: UIViewController {

    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        tableView.delegate = self
        tableView.dataSource = self
        tableView.backgroundColor = Helper.visaColor
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
}

extension ViewController: MVisaPayMerchantDelegate {

    func initiateMVisaSDKPayMerchantFlow() {
        print("Initiating MVisa Pay Merchant flow")
        
        // Init SDK with one time config
        let onetimeConfig = MVisaConfig()
        MVisaSDK.setupOneTimeConfiguration(onetimeConfig)
        
        // Card details for forming MVisaPayMerchantRequest
        let cardDetails = getCardDetail()
        let randomIndex = Int(arc4random_uniform(UInt32(cardDetails.count)))
        
        // Forming the MVisaPayMerchantRequest
        let payMerchantRequest = MVisaPayMerchantRequest(cards: cardDetails, currencyCode: "840", defaultCardIndex: randomIndex)
        
        // Launch the pay merchant flow (commented out using delegate and in place using completion block)
        //MVisaSDK.launchPayMerchantFlow(request: payMerchantRequest, delegate: self)
        MVisaSDK.launchPayMerchatFlow(request: payMerchantRequest) {
            (successful:Bool, payMerchantResponse:MVisaPayMerchantResponse?, error: MVisaError?) in
            self.payMerchantFlowDidFinish(successful, withResponse: payMerchantResponse, withError: error)
        }

    }
    
    func payMerchantFlowDidFinish(_ successful: Bool, withResponse payMerchantResponse: MVisaPayMerchantResponse?, withError error: MVisaError?) {
        print("Pay Merchnat Flow Did Finish")
        
        // Close the MVisaSDK
        MVisaSDK.endMVisaFlow()
        
        // Use the response here
        // Printing the response in logs
        let responseStr = Helper.getPayMerchantResponseStr(successful: successful, response: payMerchantResponse, error: error)
        print(responseStr)
        
        //P2M flow
        let responseVC = getResponseController()
        responseVC.responseJson = responseStr
        self.pushViewController(viewController: responseVC)
    }
    
    func initiateMVisaSDKPayMerchantAliasFlow() {
        
        // Init SDK with one time config
        let onetimeConfig = MVisaConfig()
        MVisaSDK.setupOneTimeConfiguration(onetimeConfig)
        
        // Card details for forming MVisaPayMerchantRequest
        let cardDetails = getCardDetail()
        let randomIndex = Int(arc4random_uniform(UInt32(cardDetails.count)))
        
        // Forming the MVisaPayMerchantRequest
        let payMerchantRequest = MVisaPayMerchantRequest(cards: cardDetails, currencyCode: "840", defaultCardIndex: randomIndex)
        
        // Launch the pay merchant flow (commented out using delegate and in place using completion block)
        MVisaSDK.launchPayMerchantAliasFlow(request: payMerchantRequest, aliasValidationHandler: { (aliasData: PayMerchantAliasData?, callbackHandler: PayMerchantAliasResponse?) in
            
            DispatchQueue.main.asyncAfter(deadline: .now() + 2, execute: {
                
                let resp = AliasResponse(merchantName: "Corner Store", merchantCity: "Bangalore")
                let err = AliasError(withError: "Alias not found for the given number")
                
                callbackHandler?.setData(aliasResponse: resp,
                                             error: err)
            })
            
        }) { (successful:Bool, payMerchantResponse:MVisaPayMerchantResponse?, error: MVisaError?) in
            self.payMerchantFlowDidFinish(successful, withResponse: payMerchantResponse, withError: error)
        }
    }
}

extension ViewController: MVisaPayFriendDelegate {
    func initiateMVisaSDKPayFriendFlow() {
        print("Initiating MVisa Pay Friend flow")
        
        // Init SDK with one time config
        let onetimeConfig = MVisaConfig()
        MVisaSDK.setupOneTimeConfiguration(onetimeConfig)

        // Prepare payee list to be used to create Pay Friend Request
        var payee = [MVisaPayee]()
        payee = [MVisaPayee]()
        payee.append(MVisaPayee(payeeName: "John Smith", lastFour: "1234"))
        payee.append(MVisaPayee(payeeName: "Harry Potter", lastFour: "5680"))
        payee.append(MVisaPayee(payeeName: "Sarah Blackmon", lastFour: "5739"))

        // Prepare consumer card list to be used to create Pay Friend Request
        let cards = getCardDetail()

        // Create Pay Friend Request
        let request = MVisaPayFriendRequest(cards: cards, currencyCode: "356", payees: payee, defaultCardIndex: 1)

        // Launch the pay friend flow (commented out using delegate and in place using completion block)
        //MVisaSDK.launchPayFriendFlow(request: request, delegate: self)
        MVisaSDK.launchPayFriendFlow(request: request) {
            (successful:Bool, payFriendResponse:MVisaPayFriendResponse?, error: MVisaError?) in
            self.payFriendFlowDidFinish(successful, withResponse: payFriendResponse, withError: error)
        }
    }
    
    func payFriendFlowDidFinish(_ successful: Bool, withResponse payFriendResponse: MVisaPayFriendResponse?, withError error: MVisaError?) {
        print("Pay Friend Flow Did Finish")
        
        // Close the MVisaSDK
        MVisaSDK.endMVisaFlow()
        
        guard successful else {
            print(error!.errorCodes)
            
            let alertController = UIAlertController(title: "Error", message: error?.description, preferredStyle: .alert)
            alertController.addAction(UIAlertAction(title: "OK", style: .default, handler:{ (action:UIAlertAction) in

            }))
            
            DispatchQueue.main.async(execute: {
                self.present(alertController, animated: true, completion: nil)
            })

            return
        }

        //P2M flow

        // Use the response here
        // Printing the response in logs
        let responseStr = Helper.getPayFriendResponseStr(successful: successful, response: payFriendResponse, error: error)
        print(responseStr)
        
        let responseVC = getResponseController()
        responseVC.responseJson = responseStr
        self.pushViewController(viewController: responseVC)
    }
    
    func initiatePayFriendAliasFlow() {
        
        let p = MVisaPayee(payeeName: "John Smith", lastFour: "1234", mobileNumber: "12312123", bankName: "ICICI")
        let currentPayFriendRequest = MVisaPayFriendRequest(cards: getCardDetail(), currencyCode: "356", payee: p, defaultCardIndex: 1, payType: .cardFallback, isAliasFlow: true, nameOnContactList: "John CA")
        
        MVisaSDK.launchPayFriendFlow(request: currentPayFriendRequest,
                                     completionHandler: { (successful:Bool, payFriendResponse:MVisaPayFriendResponse?, error: MVisaError?) in
            self.payFriendFlowDidFinish(successful, withResponse: payFriendResponse, withError: error)
        })
    }
    
    
    
}

extension ViewController: MVisaWithdrawCashDelegate {
    func initiateMVisaSDKWithdrawCashFlow() {
        print("Initiating MVisa Withdraw Cash flow")
        
        // Init SDK with one time config
        let onetimeConfig = MVisaConfig()
        MVisaSDK.setupOneTimeConfiguration(onetimeConfig)
        
        // Card details for forming MVisaWithdrawCashRequest
        let cardDetails = getCardDetail()
        let randomIndex = Int(arc4random_uniform(UInt32(cardDetails.count)))
        
        // Forming the MVisaWithdrawCashRequest
        let withdrawCashRequest = MVisaWithdrawCashRequest(cards: cardDetails, currencyCode: "840", defaultCardIndex: randomIndex, serviceFee: "200.00")
        
        // Launch the withdraw cash flow (commented out using delegate and in place using completion block)
        //MVisaSDK.launchWithdrawCashFlow(request: withdrawCashRequest, delegate: self)
        MVisaSDK.launchWithdrawCashFlow(request: withdrawCashRequest) {
            (successful:Bool, withdrawCashResponse: MVisaWithdrawCashResponse?, error: MVisaError?) in
            self.withdrawCashFlowDidFinish(successful, withResponse: withdrawCashResponse, withError: error)
        }
    }
    
    func withdrawCashFlowDidFinish(_ successful: Bool, withResponse withdrawCashResponse: MVisaWithdrawCashResponse?, withError error: MVisaError?) {
            print("Withdraw Cash Flow Did Finish")
            
            // Close the MVisaSDK
            MVisaSDK.endMVisaFlow()
            
            // Use the response here
            // Printing the response in logs
            let responseStr = Helper.getWithdrawCashResponseStr(successful: successful, response: withdrawCashResponse, error: error)
            print(responseStr)
        
        let responseVC = getResponseController()
        responseVC.responseJson = responseStr
        self.pushViewController(viewController: responseVC)
    }
}

extension ViewController {
    
    func getCardDetail()-> [MVisaCardDetails] {
        
        // Card details for forming MVisaWithdrawCashRequest
        let card1 = MVisaCardDetails(lastFourDigits: "1357",
                                     cardType: "SBI Debit",
                                     issuerLogo: nil,
                                     cardArtColor: nil,
                                     cardArtOverlay: nil,
                                     networkName: "Visa",
                                     networkLogo: UIImage(named:"card_network_logo_visa"))
        
        let card2 = MVisaCardDetails(lastFourDigits: "1257",
                                     cardType: "HDFC Credit",
                                     issuerLogo: nil,
                                     cardArtColor: nil,
                                     cardArtOverlay: nil,
                                     networkName: "Visa",
                                     networkLogo: nil)

        let card3 = MVisaCardDetails(lastFourDigits: "1357",
                                     cardType: "ICICI Platinum",
                                     issuerLogo: nil,
                                     cardArtColor: nil,
                                     cardArtOverlay: nil,
                                     networkName: "",
                                     networkLogo: nil)
        
        return [card1, card2, card3]
    }
    
    func getDispatchtime(forInterval timeInterval: TimeInterval) -> DispatchTime {
        return DispatchTime.now() + Double(Int64(timeInterval*Double(NSEC_PER_SEC))) / Double(NSEC_PER_SEC)
    }
    
    func getResponseController() -> ResponseJsonViewController {
        let responseVC = UIStoryboard(name: "Main", bundle: Bundle.main).instantiateViewController(withIdentifier: "ResponseJsonViewController") as! ResponseJsonViewController
        
        return responseVC
    }
    
    func pushViewController(viewController: UIViewController) {
        
        weak var weakSelf = self
        DispatchQueue.main.asyncAfter(deadline: getDispatchtime(forInterval: 0.25), execute: {
            weakSelf?.navigationController?.pushViewController(viewController, animated: true)
        })
    }
}




















